-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftCastBarOverlay")
local SML = AceLibrary("SharedMedia-1.0")
local SML_statusbar = SML:List("statusbar")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Cast Bar Overlay"] = true,
	["Cast bar overlay options"] = true,
	["Enable"] = true,
	["Show an overlay on the full cast bar"] = true,
	["Color"] = true,
	["Sets the cast bar overlay color"] = true,
	["Texture"] = true,
	["Sets the cast bar overlay texture"] = true,
	["Horizontal Padding"] = true,
	["Horizontal padding of the overlay texture compared to the the cast bar"] = true,
	["Vertical Padding"] = true,
	["Vertical padding of the overlay texture compared to the the cast bar"] = true,
	["Emulate Background"] = true,	
	["Dynamically adjust the position of the cast bar overlay texture to make it appear as a background to the cast bar"] = true,
} end)

-----------------------------------------------------------------------------

local AloftCastBarOverlay = Aloft:NewModule("CastBarOverlay", "AceEvent-2.0")
AloftCastBarOverlay.db = Aloft:AcquireDBNamespace("castBarOverlay")
Aloft:RegisterDefaults("castBarOverlay", "profile", {
	enable				= true,
	color				= { 0.25, 0.25, 0.25, 0.5 },
	texture				= "Blizzard",
	horizontalPadding	= 0,
	verticalPadding		= 0,
	emulateBackground	= true,
})

Aloft.Options.args.castBarOverlay =
{
	type = 'group',
	name = L["Cast Bar Overlay"],
	desc = L["Cast bar overlay options"],
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show an overlay on the full cast bar"],
			get = function() return AloftCastBarOverlay.db.profile.enable end,
			set = function(v) AloftCastBarOverlay.db.profile.enable = v AloftCastBarOverlay:UpdateAll() end
		},
		color =
		{
			type = 'color',
			name = L["Color"],
			desc = L["Sets the cast bar overlay color"],
			get = function() return unpack(AloftCastBarOverlay.db.profile.color) end,
			set = function(r, g, b, a) AloftCastBarOverlay.db.profile.color = { r, g, b, a } AloftCastBarOverlay:UpdateAll() end,
			hasAlpha = true
		},
		texture =
		{
			type = "text", 
			name = L["Texture"], 
			desc = L["Sets the cast bar overlay texture"],
			get = function() return AloftCastBarOverlay.db.profile.texture end,
			set = function(v) AloftCastBarOverlay.db.profile.texture = v AloftCastBarOverlay:UpdateAll() end,
			validate = SML_statusbar,
		},
		horizontalPadding =
		{
			type = 'range',
			name = L["Horizontal Padding"],
			desc = L["Horizontal padding of the overlay texture compared to the the cast bar"],
			min = 0,
			max = 8,
			step = 0.1,
			disabled = function() return AloftCastBarOverlay.db.profile.emulateBackground end,
			get = function() return AloftCastBarOverlay.db.profile.horizontalPadding end,
			set = function(v) AloftCastBarOverlay.db.profile.horizontalPadding = v AloftCastBarOverlay:UpdateAll() end
		},
		verticalPadding =
		{
			type = 'range',
			name = L["Vertical Padding"],
			desc = L["Vertical padding of the overlay texture compared to the the cast bar"],
			min = 0,
			max = 8,
			step = 0.1,
			disabled = function() return AloftCastBarOverlay.db.profile.emulateBackground end,
			get = function() return AloftCastBarOverlay.db.profile.verticalPadding end,
			set = function(v) AloftCastBarOverlay.db.profile.verticalPadding = v AloftCastBarOverlay:UpdateAll() end
		},
		emulateBackground = 
		{
			type = 'toggle',
			name = L["Emulate Background"],
			desc = L["Dynamically adjust the position of the cast bar overlay texture to make it appear as a background to the cast bar"],
			get = function() return AloftCastBarOverlay.db.profile.emulateBackground end,
			set = function(v) AloftCastBarOverlay.db.profile.emulateBackground = v AloftCastBarOverlay:UpdateAll() end
		},
	},
}

-----------------------------------------------------------------------------

function AloftCastBarOverlay:UpdateAll()
	for frame in Aloft:IterateNameplates() do	
		self:SetupFrame(frame)

		local castBar = frame.aloftData.castBar
		if castBar:IsShown() then
			self:OnCastBarShow(frame)
		end
		
		self:Update(frame)
	end
end

-----------------------------------------------------------------------------

function AloftCastBarOverlay:OnEnable()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	self:RegisterEvent("Aloft:OnCastBarShow", "OnCastBarShow")
	self:RegisterEvent("Aloft:OnCastBarValueChanged", "Update")

	self:RegisterEvent("SharedMedia_SetGlobal", function(mediatype, override)
		if mediatype == "statusbar" then
			self:UpdateAll()
		end
	end)
end

function AloftCastBarOverlay:SetupFrame(frame)
	local castBarOverlayFrame = select(2, frame:GetRegions())
	
	castBarOverlayFrame:ClearAllPoints()
	castBarOverlayFrame:SetTexture(SML:Fetch("statusbar", self.db.profile.texture))
	castBarOverlayFrame:SetTexCoord(0.0, 1.0, 0.0, 1.0)

	if self.db.profile.enable and frame.aloftData.castBar:IsShown() then	
		castBarOverlayFrame:Show()
	else
		castBarOverlayFrame:Hide()
	end

	frame.aloftData.castBarOverlayFrame = castBarOverlayFrame
end

function AloftCastBarOverlay:OnCastBarShow(frame)
	local castBarOverlayFrame = frame.aloftData.castBarOverlayFrame

	if self.db.profile.enable then
		-- If we try to make castBarOverlay reference castBar, the game just blows up.
		-- No crash, no lua errors - the game just disappears.
		--
		-- So it's ugly, but we're going to directly access another core module's variables to avoid this
		local healthBar = frame.aloftData.healthBar
		local castBarDB = Aloft:AcquireDBNamespace("castBar").profile
	
		castBarOverlayFrame:ClearAllPoints()
		castBarOverlayFrame:SetPoint("TOPLEFT", healthBar, "BOTTOMLEFT", castBarDB.offsets.left - self.db.profile.horizontalPadding, castBarDB.offsets.vertical + self.db.profile.verticalPadding)
		castBarOverlayFrame:SetPoint("BOTTOMRIGHT", healthBar, "BOTTOMRIGHT", castBarDB.offsets.right + self.db.profile.horizontalPadding, castBarDB.offsets.vertical - self.db.profile.verticalPadding - castBarDB.height)
		castBarOverlayFrame:SetVertexColor(unpack(self.db.profile.color))		

		if self.db.profile.emulateBackground then
			self:Update(frame)
		end
	else
		castBarOverlayFrame:SetAlpha(0)
		castBarOverlayFrame:Hide()
	end
end

function AloftCastBarOverlay:Update(frame)
	local castBarOverlayFrame = frame.aloftData.castBarOverlayFrame

	castBarOverlayFrame:SetVertexColor(unpack(self.db.profile.color))
	
	if self.db.profile.emulateBackground then
		local castBar = frame.aloftData.castBar
		local minValue, maxValue = castBar:GetMinMaxValues()

		-- Just as the cast bar disappears, both seem to be set to 0.
		if minValue ~= maxValue then
			local k = (castBar:GetValue() - minValue) / (maxValue - minValue)
			if k == 1 then
				castBarOverlayFrame:SetAlpha(0)
				castBarOverlayFrame:Hide()
			else
				-- If we try to make castBarOverlay reference castBar, the game just blows up.
				-- No crash, no lua errors - the game just disappears.
				--
				-- So it's ugly, but we're going to directly access another core module's variables to avoid this
				local healthBar = frame.aloftData.healthBar
				local castBarDB = Aloft:AcquireDBNamespace("castBar").profile

				-- Why does this produce a value that's way out?
				-- castBar:GetScale() is 1, not sure why this doesn't work.
				-- local castBarWidth = castBar:GetWidth()
				local castBarWidth = healthBar:GetWidth() + castBarDB.offsets.right - castBarDB.offsets.left

				castBarOverlayFrame:ClearAllPoints()
				castBarOverlayFrame:SetPoint("TOPRIGHT", healthBar, "BOTTOMRIGHT", castBarDB.offsets.right - self.db.profile.horizontalPadding, castBarDB.offsets.vertical + self.db.profile.verticalPadding)
				castBarOverlayFrame:SetWidth((1.0-k) * castBarWidth)
				castBarOverlayFrame:SetHeight(castBarDB.height)
				castBarOverlayFrame:SetTexCoord(k, 1.0, 0.0, 1.0)
				castBarOverlayFrame:Show()
			end
		end
	end
end

-----------------------------------------------------------------------------
