-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftHealthBar")
local SML = AceLibrary("SharedMedia-1.0")
local SML_statusbar = SML:List("statusbar")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Health Bar"] = true,
	["Health Bar options"] = true,
	["Width"] = true,
	["Sets the width of the health bar"] = true,
	["Height"] = true,
	["Sets the height of the health bar"] = true,
	["Texture"] = true,
	["Sets the health bar texture"] = true,
	["Alpha"] = true,
	["Sets the health bar alpha"] = true,
	["Color by Class"] = true,
	["Sets the health bar color according to class."] = true,
	["Group Only"] = true,
	["Only colors group member's health bars by class"] = true,
	["Colors"] = true,
	["Customize colors"] = true,
	["Friendly Players"] = true,
	["Sets the health bar color for friendly players"] = true,
	["Friendly NPCs"] = true,
	["Sets the health bar color for friendly NPC Bosses"] = true,
	["Friendly Boss"] = true,
	["Sets the health bar color for friendly NPCs"] = true,
	["Hostile Units"] = true,
	["Sets the health bar color for hostile units"] = true,
	["Hostile Boss"] = true,
	["Sets the health bar color for hostile boss units"] = true,
	["Neutral Units"] = true,
	["Sets the health bar color for neutral units"] = true,
	["Group Pets"] = true,
	["Sets the health bar color for group pets"] = true,

} end)

-----------------------------------------------------------------------------

local AloftHealthBar = Aloft:NewModule("HealthBar", "AceEvent-2.0")
AloftHealthBar.db = Aloft:AcquireDBNamespace("healthBar")
Aloft:RegisterDefaults("healthBar", "profile", {
	texture			= "Blizzard",
	alpha			= 0.9,
	colorByClass	= true,
	groupOnly		= false,
	width			= 115,
	height			= 11,
	colors =
	{
		friendlyPlayer	= { 0, 0, 1 },
		friendlyNPC		= { 0, 1, 0 },
		friendlyBoss	= { 0, 1, 0 },
		hostile			= { 1, 0, 0 },
		hostileBoss		= { 1, 0, 0 },
		neutral			= { 1, 1, 0 },
		groupPet		= { 1, 0.5, 1 },
	},
})

Aloft.Options.args.healthBar =
{
	type = 'group',
	name = L["Health Bar"],
	desc = L["Health Bar options"],
	args =
	{
		texture =
		{
			type = "text", 
			name = L["Texture"], 
			desc = L["Sets the health bar texture"],
			get = function() return AloftHealthBar.db.profile.texture end,
			set = function(v) AloftHealthBar.db.profile.texture = v AloftHealthBar:UpdateAll() end,
			validate = SML_statusbar,
		},
		width =
		{
			type = 'range',
			name = L["Width"],
			desc = L["Sets the width of the health bar"],
			min = 50,
			max = 250,
			step = 1,
			get = function() return AloftHealthBar.db.profile.width end,
			set = function(v) AloftHealthBar.db.profile.width = v AloftHealthBar:UpdateSizesAll() end,
		},
		height =
		{
			type = 'range',
			name = L["Height"],
			desc = L["Sets the height of the health bar"],
			min = 4,
			max = 40,
			step = 1,
			get = function() return AloftHealthBar.db.profile.height end,
			set = function(v) AloftHealthBar.db.profile.height = v AloftHealthBar:UpdateSizesAll() end,
		},
		alpha =
		{
			type = 'range',
			name = L["Alpha"],
			desc = L["Sets the health bar alpha"],
			min = 0,
			max = 1.0,
			step = 0.05,
			get = function() return AloftHealthBar.db.profile.alpha end,
			set = function(v) AloftHealthBar.db.profile.alpha = v AloftHealthBar:UpdateAll() end
		},
		colorByClass =
		{
			type = 'toggle',
			name = L["Color by Class"],
			desc = L["Sets the health bar color according to class."],
			get = function() return AloftHealthBar.db.profile.colorByClass end,
			set = function(v) AloftHealthBar.db.profile.colorByClass = v AloftHealthBar:UpdateAll() end
		},
		groupOnly =
		{
			type = 'toggle',
			name = L["Group Only"],
			desc = L["Only colors group member's health bars by class"],
			disabled = function() return not AloftHealthBar.db.profile.colorByClass end,
			get = function() return AloftHealthBar.db.profile.groupOnly end,
			set = function(v) AloftHealthBar.db.profile.groupOnly = v AloftHealthBar:UpdateAll() end
		},
		colors =
		{
			type = 'group',
			name = L["Colors"],
			desc = L["Customize colors"],
			args =
			{
				friendlyPlayer =
				{
					type = 'color',
					name = L["Friendly Players"],
					desc = L["Sets the health bar color for friendly players"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.friendlyPlayer) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.friendlyPlayer = { r, g, b } AloftHealthBar:UpdateAll() end
				},
				friendlyNPC =
				{
					type = 'color',
					name = L["Friendly NPCs"],
					desc = L["Sets the health bar color for friendly NPCs"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.friendlyNPC) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.friendlyNPC = { r, g, b } AloftHealthBar:UpdateAll() end
				},
				friendlyBoss =
				{
					type = 'color',
					name = L["Friendly Boss"],
					desc = L["Sets the health bar color for friendly NPC Bosses"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.friendlyBoss) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.friendlyBoss = { r, g, b } AloftHealthBar:UpdateAll() end
				},
				hostile =
				{
					type = 'color',
					name = L["Hostile Units"],
					desc = L["Sets the health bar color for hostile units"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.hostile) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.hostile = { r, g, b } AloftHealthBar:UpdateAll() end
				},
				hostileBoss =
				{
					type = 'color',
					name = L["Hostile Boss"],
					desc = L["Sets the health bar color for hostile boss units"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.hostileBoss) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.hostileBoss = { r, g, b } AloftHealthBar:UpdateAll() end
				},
				neutral =
				{
					type = 'color',
					name = L["Neutral Units"],
					desc = L["Sets the health bar color for neutral units"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.neutral) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.neutral = { r, g, b } AloftHealthBar:UpdateAll() end
				},
				groupPet =
				{
					type = 'color',
					name = L["Group Pets"],
					desc = L["Sets the health bar color for group pets"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.groupPet) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.groupPet = { r, g, b } AloftHealthBar:UpdateAll() end
				},
			}
		}
	},
}

-----------------------------------------------------------------------------

function AloftHealthBar:UpdateAll()
	for frame in Aloft:IterateNameplates() do
		self:SetupFrame(frame)

		if frame:IsShown() then
			self:Update(frame)
		end
	end

	if self.db.profile.colorByClass then
		self:RegisterEvent("Aloft:OnClassDataChanged", "Update")
	else
		self:CancelEvent("Aloft:OnClassDataChanged")
	end
end

function AloftHealthBar:UpdateSizesAll()
	for frame in Aloft:IterateNameplates() do
		self:OnNameplateShow(frame)
	end
	
	for frame in Aloft:IterateVisibleNameplates() do
		self:TriggerEvent("Aloft:OnHealthBarValueChanged", frame)
	end
end

function AloftHealthBar:UpdateForName(name)
	local frame = Aloft:GetNameplateFrame(name)
	if frame then
		AloftHealthBar:Update(frame)
	end
end

-----------------------------------------------------------------------------

function AloftHealthBar:SetFrameSize(frame)
	-- This is a mechanism used to ensure that our bars are clickable...
	frame:SetHeight(self.db.profile.height+8)
	frame:SetWidth(self.db.profile.width+8)
end

function AloftHealthBar:OnEnable()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	self:RegisterEvent("Aloft:OnNameplateShow", "OnNameplateShow")
	self:RegisterEvent("Aloft:OnHealthBarValueChanged", "Update")
	self:RegisterEvent("Aloft:OnHealthBarColorChanged", "Update")

	self:RegisterEvent("SharedMedia_SetGlobal", function(mediatype, override)
		if mediatype == "statusbar" then
			self:UpdateAll()
		end
	end)
	
	if self.db.profile.colorByClass then
		self:RegisterEvent("Aloft:OnClassDataChanged", "Update")
	end
end

function AloftHealthBar:SetupFrame(frame)
	local healthBar = frame.aloftData.healthBar
	healthBar:SetStatusBarTexture(SML:Fetch("statusbar", self.db.profile.texture))
end

function AloftHealthBar:OnNameplateShow(frame)
	local healthBar = frame.aloftData.healthBar

	healthBar:SetWidth(self.db.profile.width)
	healthBar:SetHeight(self.db.profile.height)

	self:Update(frame)	
	
	self:SetFrameSize(frame, healthBar)
end

function AloftHealthBar:Update(frame)
	local aloftData = frame.aloftData
	local healthBar = aloftData.healthBar

	if self.db.profile.colorByClass then
		local class = aloftData.class
		if class then
			local color = RAID_CLASS_COLORS[class]
			if color then
				healthBar:SetStatusBarColor(color.r, color.g, color.b, self.db.profile.alpha)
				return
			end
		end
	end

	local color = self.db.profile.colors[aloftData.type]
	if color then
		healthBar:SetStatusBarColor(color[1], color[2], color[3], self.db.profile.alpha)
	else
		local r, g, b = healthBar:GetStatusBarColor()
		healthBar:SetStatusBarColor(r, g, self.db.profile.alpha)
	end
end

-----------------------------------------------------------------------------
