-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftLevelText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Level Text"] = true,
	["Level text options"] = true,
	["Enable"] = true,
	["Enable level text display on the nameplate"] = true,
	["Font"] = true,
	["Sets the font for level text"] = true,
	["Font Size"] = true,
	["Sets the font height of the level text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on level text"] = true,
	["Outline"] = true,
	["Sets the font outline of the level text"] = true,
	["Anchor"] = true, 
	["Sets the anchor for the level text"] = true,
	["Anchor To"] = true, 
	["Sets the relative point on the health bar to anchor the level text"] = true,
	["X Offset"] = true,
	["X offset of the level text"] = true,
	["Y Offset"] = true,
	["Y offset of the level text"] = true,

	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,
} end)

-----------------------------------------------------------------------------

local AloftLevelText = Aloft:NewModule("LevelText", "AceEvent-2.0")
AloftLevelText.db = Aloft:AcquireDBNamespace("levelText")
Aloft:RegisterDefaults("levelText", "profile", {
	enable			= true,
	font			= "Arial Narrow",
	fontSize		= 8,
	outline			= "OUTLINE",
	point			= "RIGHT",
	relativeToPoint	= "RIGHT",
	offsetX			= 0,
	offsetY			= 0,
})

Aloft.Options.args.levelText =
{
	type = 'group',
	name = L["Level Text"],
	desc = L["Level text options"],
	args =
	{
		enable = 
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Enable level text display on the nameplate"],
			get = function() return AloftLevelText.db.profile.enable end,
			set = function(v) AloftLevelText.db.profile.enable = v AloftLevelText:UpdateAll() end
		},
		font =
		{
			type = "text", 
			name = L["Font"], 
			desc = L["Sets the font for level text"],
			get = function() return AloftLevelText.db.profile.font end,
			set = function(v) AloftLevelText.db.profile.font = v AloftLevelText:UpdateAll() end,
			validate = SML_font,
		},
		fontSize =
		{
			type = 'range',
			name = L["Font Size"],
			desc = L["Sets the font height of the level text"],
			max = 16,
			min = 5,
			step = 1,
			get = function() return AloftLevelText.db.profile.fontSize end,
			set = function(value) AloftLevelText.db.profile.fontSize = value AloftLevelText:UpdateAll() end
		},
		shadow =
		{
			type = 'toggle',
			name = L["Font Shadow"],
			desc = L["Show font shadow on level text"],
			get = function() return AloftLevelText.db.profile.shadow end,
			set = function(v) AloftLevelText.db.profile.shadow = v AloftLevelText:UpdateAll() end
		},
		outline =
		{
			type = 'text',
			name = L["Outline"],
			desc = L["Sets the font outline of the level text"],
			get = function() return AloftLevelText.db.profile.outline end,
			set = function(value) AloftLevelText.db.profile.outline = value AloftLevelText:UpdateAll() end,
			validate = { [""] = L["None"], ["OUTLINE"] = L["Normal"], ["THICKOUTLINE"] = L["Thick"] },
		},
		anchor = {
			type = "text", 
			name = L["Anchor"], 
			desc = L["Sets the anchor for the level text"],
			get = function() return AloftLevelText.db.profile.point end,
			set = function(v) AloftLevelText.db.profile.point = v AloftLevelText:UpdateAll() end,
			validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
		},
		anchorto = {
			type = "text", 
			name = L["Anchor To"], 
			desc = L["Sets the relative point on the health bar to anchor the level text"],
			get = function() return AloftLevelText.db.profile.relativeToPoint end,
			set = function(v) AloftLevelText.db.profile.relativeToPoint = v AloftLevelText:UpdateAll() end,
			validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
		},
		offsetX =
		{
			type = 'range',
			name = L["X Offset"],
			desc = L["X offset of the level text"],
			min = -32,
			max = 32,
			step = 1,
			get = function() return AloftLevelText.db.profile.offsetX end,
			set = function(v) AloftLevelText.db.profile.offsetX = v AloftLevelText:UpdateAll() end
		},
		offsetY =
		{
			type = 'range',
			name = L["Y Offset"],
			desc = L["Y offset of the level text"],
			min = -32,
			max = 32,
			step = 1,
			get = function() return AloftLevelText.db.profile.offsetY end,
			set = function(v) AloftLevelText.db.profile.offsetY = v AloftLevelText:UpdateAll() end
		},
	},
}

-----------------------------------------------------------------------------

function AloftLevelText:UpdateAll()
	for frame in Aloft:IterateNameplates() do
		local levelTextFrame = frame.aloftData.levelTextFrame
	
		if self.db.profile.enable then
			levelTextFrame:Show()
		else
			levelTextFrame:Hide()
		end
		self:PrepareText(levelTextFrame, self.db.profile)
		self:PlaceFrame(levelTextFrame, frame.aloftData.healthBar, self.db.profile)
	end
	
	if self.db.profile.enable then
		self:CancelEvent("Aloft:OnNameplateShow")
	else
		self:RegisterEvent("Aloft:OnNameplateShow", "OnNameplateShow")
	end
end

-----------------------------------------------------------------------------

function AloftLevelText:OnEnable()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	
	if not self.db.profile.enable then
		self:RegisterEvent("Aloft:OnNameplateShow", "OnNameplateShow")
	end
end

function AloftLevelText:SetupFrame(frame)
	local levelTextFrame = select(6, frame:GetRegions())

	frame.aloftData.levelTextFrame = levelTextFrame
	self:PrepareText(levelTextFrame, self.db.profile)
	self:PlaceFrame(levelTextFrame, frame.aloftData.healthBar, self.db.profile)
end

function AloftLevelText:OnNameplateShow(frame)
	frame.aloftData.levelTextFrame:Hide()
end

-----------------------------------------------------------------------------
